@@ -83,8 +83,8 @@ class ConfigFileParser(object):
             stream: A config file input stream (such as an open file object).
 
         Returns:
-            OrderedDict of items where the keys have type string and the
-            values have type either string or list (eg. to support config file
+            OrderedDict: Items where the keys are strings and the
+            values are either strings or lists (eg. to support config file
             formats like YAML which allow lists).
         """
         raise NotImplementedError("parse(..) not implemented")
@@ -95,8 +95,8 @@ class ConfigFileParser(object):
 
         Args:
             items: an OrderedDict of items to be converted to the config file
-            format. Keys should be strings, and values should be either strings
-            or lists.
+                format. Keys should be strings, and values should be either strings
+                or lists.
 
         Returns:
             Contents of config file as a string
@@ -110,7 +110,7 @@ class ConfigFileParserException(Exception):
 
 class DefaultConfigFileParser(ConfigFileParser):
     """Based on a simplified subset of INI and YAML formats. Here is the
-    supported syntax:
+    supported syntax::
 
 
         # this is a comment
@@ -257,7 +257,7 @@ class ConfigparserConfigFileParser(ConfigFileParser):
             strict=True,
             empty_lines_in_values=False,
         )
-        items = {"DEFAULT":items}
+        items = {"DEFAULT": items}
         config.read_dict(items)
         stream = io.StringIO()
         config.write(stream)
@@ -342,10 +342,10 @@ class ArgumentParser(argparse.ArgumentParser):
 
     def __init__(self, *args, **kwargs):
 
-        """Supports args of the argparse.ArgumentParser constructor
-        as **kwargs, as well as the following additional args.
+        r"""Supports args of the argparse.ArgumentParser constructor
+        as \*\*kwargs, as well as the following additional args.
 
-        Additional Args:
+        Arguments:
             add_config_file_help: Whether to add a description of config file
                 syntax to the help message.
             add_env_var_help: Whether to add something to the help message for
@@ -361,12 +361,14 @@ class ArgumentParser(argparse.ArgumentParser):
                 taking precedence over previous ones. This allows an application
                 to look for config files in multiple standard locations such as
                 the install directory, home directory, and current directory.
-                Also, shell * syntax can be used to specify all conf files in a
-                directory. For example:
-                ["/etc/conf/app_config.ini",
-                 "/etc/conf/conf-enabled/*.ini",
-                "~/.my_app_config.ini",
-                "./app_config.txt"]
+                Also, shell \* syntax can be used to specify all conf files in a
+                directory. For example::
+
+                    ["/etc/conf/app_config.ini",
+                    "/etc/conf/conf-enabled/*.ini",
+                    "~/.my_app_config.ini",
+                    "./app_config.txt"]
+
             ignore_unknown_config_file_keys: If true, settings that are found
                 in a config file but don't correspond to any defined
                 configargparse args will be ignored. If false, they will be
@@ -449,7 +451,7 @@ class ArgumentParser(argparse.ArgumentParser):
         """Supports all the same args as the ArgumentParser.parse_args(..),
         as well as the following additional args.
 
-        Additional Args:
+        Arguments:
             args: a list of args as in argparse, or a string (eg. "-x -y bla")
             config_file_contents: String. Used for testing.
             env_vars: Dictionary. Used for testing.
@@ -476,7 +478,7 @@ class ArgumentParser(argparse.ArgumentParser):
         """Supports all the same args as the ArgumentParser.parse_args(..),
         as well as the following additional args.
 
-        Additional Args:
+        Arguments:
             args: a list of args as in argparse, or a string (eg. "-x -y bla")
             config_file_contents (str). Used for testing.
             env_vars (dict). Used for testing.
@@ -639,6 +641,20 @@ class ArgumentParser(argparse.ArgumentParser):
         self.write_config_file(namespace, output_file_paths, exit_after=True)
         return namespace, unknown_args
 
+    def get_source_to_settings_dict(self):
+        """If called after parse_args() or parse_known_args(), this dict. will contain up to 4 keys corresponding
+        to where a given option's value is coming from:
+            "command_line"
+            "environment_variables"
+            "config_file"
+            "defaults"
+        Each such key, will be mapped to another dictionary containing the options set via that method. Here the key
+        will be the option name, and the value will be a 2-tuple of the form (argparse Action obj, string value).
+        """
+
+        return self._source_to_settings
+
+
     def write_config_file(self, parsed_namespace, output_file_paths, exit_after=False):
         """Write the given settings to output files.
 
@@ -663,12 +679,10 @@ class ArgumentParser(argparse.ArgumentParser):
             for output_file_path in output_file_paths:
                 with self._config_file_open_func(output_file_path, "w") as output_file:
                     output_file.write(file_contents)
-            message = "Wrote config file to " + ", ".join(output_file_paths)
-            if exit_after:
-                self.exit(0, message)
-            else:
-                print(message)
 
+            print("Wrote config file to " + ", ".join(output_file_paths))
+            if exit_after:
+                self.exit(0)
 
     def get_command_line_key_for_unknown_config_file_setting(self, key):
         """Compute a commandline arg key to be used for a config file setting
@@ -692,7 +706,7 @@ class ArgumentParser(argparse.ArgumentParser):
             source_to_settings: the dictionary described in parse_known_args()
             parsed_namespace: namespace object created within parse_known_args()
         Returns:
-            an OrderedDict where keys are strings and values are either strings
+            OrderedDict: where keys are strings and values are either strings
             or lists
         """
         config_file_items = OrderedDict()
@@ -811,7 +825,6 @@ class ArgumentParser(argparse.ArgumentParser):
 
         return keys
 
-
     def _open_config_files(self, command_line_args):
         """Tries to parse config file path(s) from within command_line_args.
         Returns a list of opened config files, including files specified on the
@@ -981,7 +994,7 @@ def add_argument(self, *args, **kwargs):
     This method supports the same args as ArgumentParser.add_argument(..)
     as well as the additional args below.
 
-    Additional Args:
+    Arguments:
         env_var: If set, the value of this environment variable will override
             any config file or default values for this arg (but can itself
             be overridden on the commandline). Also, if auto_env_var_prefix is
